# listx.tcl
#
# 	This file implements a set of procs for enhanced list functions
#
# list::lsearchAll	- Return a list of list members that match a glob
#
#####################################################################
# This code is written and copyrighted by Clif Flynt, 1998
#
# It is licenced for public use with these terms:
#
#  This licence agreement must remain intact with the code.
# 
#  You may use this code freely for non-commercial purposes.
#
#  You may examine and modify the code to your heart's content.
#
#  You may not claim to have authored the code (aside from modifications
#  you may have made.)
#
#  You may not distribute the code without this license agreement
#
#  You may not sell this code or distribute it with a commercial product
#      without arrangements with Clif Flynt.
#
#  Maintenance, extension, modification, etc can be performed by:
#
#   Flynt Consulting Services,
#   9300 Fleming Rd.
#   Dexter, MI  48130
#
#   Contact: clif@cflynt.com
#####################################################################

package provide listx 1.0

##############################################################################
#  trimList --
# 	Trims each entry of a list, returns the clean list
# Arguments
#	lst	- The list
#	trimval	- Characters to trim.  defaults to whitespace.
# Results:
#	Returns a trimmed list, each entry has no garbage fore or aft.
#

  proc trimList {lst {trimval "	 \n\r"}} {
      set rtn ""
      foreach l $lst {
          lappend rtn [string trim $l $trimval]
      }
  return $rtn
  }

##############################################################################
#  lsearchAll --
# 	Finds each member of a list that matches a glob pattern
# Arguments
#	lst	- The list
#	glob    - The glob pattern to search for
# Results:
#	Returns a list of all $lst entries that string match $glob.
#

proc lsearchAll {lst glob} {
    set rtn ""
    foreach l $lst {
        if {[string match $glob $l]} {
        lappend rtn $l
	}
    }
return $rtn
}

##############################################################################
#  lsearchAllPos --
# 	Finds each member of a list that matches a glob pattern
# Arguments
#	lst	- The list
#	glob    - The glob pattern to search for
# Results:
#	Returns a list of all $lst positions that string match $glob.
#

proc lsearchAllPos {lst glob} {
    set rtn ""
    set i 0;
    foreach l $lst {
        if {[string match $glob $l]} {
            lappend rtn $i
	}
	incr i;
    }
return $rtn
}


##############################################################################
#  splitMulti --
# 	Splits the input on a regular expression
# Arguments
#	regExp	- The regular expression
#	text	- The text	
# Results:
#	Returns a split list, or a null string if there is no valid separator
#       in the string.

  proc splitMulti {regExp text} {

  set seplist [list * ! @ # % ^ & ( ) + = - _ , | ~ ` a b q r x y z A B Q R X Y Z ]
  set match 0;

  foreach sep $seplist {
      if {[string first $sep $text] < 0} {set match 1; break}
  }

  if {!$match} {
      error "NO MATCH TO SPLIT ON"
      return ""
  }
  
  regsub -all -nocase -- $regExp $text $sep dmy

  ;# Get rid of empty list elements at front and ?end?
  set dmy [string trim $dmy $sep]
  # puts "sep: $sep dmy: ...$dmy..[split $dmy $sep].."
  return [split $dmy $sep]
  }

################################################################
# proc getListElement {lst glob}--
#    Returns the list element that contains glob pattern 'glob' or
#    empty string.
# Arguments
#   lst		The list to search for a list element
#   glob	A glob pattern to search for.
#
# Results
#   Returns a subset of $lst or ""
# 
proc getListElement {lst glob} {
    set pos [lsearch $lst *$glob*]
# puts "glob: $glob lst: $lst -- glob: $glob"

# puts "Pos: $pos -- [lindex $lst $pos]"
    return [lindex $lst $pos]
}
